# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "02/12/2021"


from tomwer.core.scan.hdf5scan import HDF5TomoScan
from tomwer.core.utils.nxtomoutils import get_n_series
from tomoscan.esrf.hdf5scan import ImageKey
from silx.gui import qt


def check_dark_series(scan: HDF5TomoScan, logger=None, user_input: bool = True) -> bool:
    """
    check:
        - if scan has computed_dark attached. If there is more than one then ask confirmation to the user to process the dataset
        - else if the NXTomo has more than one serie of dark. Otherwise ask confirmation to the user to process the dataset

    ask confirmation for processing is done by a pop up dialog.

    :param HDF5TomoScan scan: scan to check
    :param bool user_input: if True and if n series of flat is invalid then ask the user to confirm the processing. Else print a warning and refuse processing

    :return: True if processing can be done else False
    :note: user can 'force' the processing to be done
    """
    if not isinstance(scan, HDF5TomoScan):
        raise TypeError(
            f"scan is expected to be an instance of {HDF5TomoScan} not a {type(scan)}"
        )
    try:
        image_keys = scan.image_keys
    except AttributeError:
        image_keys = scan.image_key

    n_series = get_n_series(
        image_key_values=image_keys, image_key_type=ImageKey.DARK_FIELD
    )
    if n_series != 1:
        text = f"{n_series} serie(s) of dark found when one expected. Nabu won't be able to run reconstruction (flat field will fail)."
        if n_series > 1:
            text += "\nYou can edit image_keys from the `image-key-editor` to ignore a serie (invalid all keys of series you want to ignore)."
        if user_input is False:
            text += "\n skip processing"
            if logger is not None:
                logger.error(text)
            return False
        else:
            msg = qt.QMessageBox()
            msg.setIcon(qt.QMessageBox.Warning)
            types = qt.QMessageBox.Ok | qt.QMessageBox.Cancel
            msg.setStandardButtons(types)
            text += "\nContinue ?"
            msg.setText(text)
            return msg.exec_() == qt.QMessageBox.Ok
    else:
        return True


def check_flat_series(scan: HDF5TomoScan, logger, user_input: bool = True) -> bool:
    """
    Insure the scan contains at least one serie of flat. Otherwise warn the user nabu will not be able to process
    """
    if not isinstance(scan, HDF5TomoScan):
        raise TypeError(
            f"scan is expected to be an instance of {HDF5TomoScan} not a {type(scan)}"
        )
    try:
        image_keys = scan.image_keys
    except AttributeError:
        image_keys = scan.image_key
    n_series = get_n_series(
        image_key_values=image_keys, image_key_type=ImageKey.FLAT_FIELD
    )
    if n_series == 0:
        text = "No flat found. Nabu will no be able to do the flat field correction and to reconstruct dataset."
        if user_input is False:
            if logger is not None:
                logger.error(text)
            return False
        else:
            msg = qt.QMessageBox()
            msg.setIcon(qt.QMessageBox.Warning)
            types = qt.QMessageBox.Ok | qt.QMessageBox.Cancel
            msg.setStandardButtons(types)
            text += "\nContinue ?"
            msg.setText(text)
            return msg.exec_() == qt.QMessageBox.Ok
    else:
        return True



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["C. Nemoz", "H. Payno"]
__license__ = "MIT"
__date__ = "01/08/2018"

from orangewidget import widget, gui
from orangewidget.widget import Input
from tomwer.core.scan.scanbase import TomwerScanBase
from tomwer.gui.stacks import RadioStack
import tomwer.core.process.visualization.radiostack
import logging

logger = logging.getLogger(__name__)


class RadioStackOW(widget.OWBaseWidget, openclass=True):
    """
    This widget will make stack radios incoming and allow user to browse into
    it.
    """

    name = "radio stack"
    id = "orange.widgets.tomwer.slicesstack.radiostack"
    description = (
        "This widget will save all scan path given to here "
        "and extract received radio files with there shortest"
        "unique basename to be able to browse them"
    )

    icon = "icons/radiosstack.svg"
    priority = 27
    keywords = ["tomography", "radio", "tomwer", "stack", "group"]

    ewokstaskclass = tomwer.core.process.visualization.radiostack._RadioStackPlaceHolder

    allows_cycle = True
    compress_signal = False

    want_main_area = True
    resizing_enabled = True

    class Inputs:
        data = Input(name="data", type=TomwerScanBase)

    def __init__(self, parent=None):
        widget.OWBaseWidget.__init__(self, parent)
        self._box = gui.vBox(self.mainArea, self.name)
        self._viewer = RadioStack(parent=self)
        self._box.layout().addWidget(self._viewer)

    @Inputs.data
    def addLeafScan(self, scanID):
        if scanID is None:
            return
        self._viewer.addLeafScan(scanID)



# coding: utf-8
###########################################################################
# Copyright (C) 2016-2019 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
#############################################################################

__authors__ = ["H.Payno"]
__license__ = "MIT"
__date__ = "13/08/2021"

import shutil

from processview.core.manager import ProcessManager
from tomwer.core.scan.scanbase import TomwerScanBase
from tomwer.core.settings import get_lbsram_path, isOnLbsram
from tomwer.core.utils import isLowOnMemory
from processview.core.manager import DatasetState
from ..processingstack import FIFO, ProcessingThread
from processview.core.superviseprocess import SuperviseProcess
from tomwer.core.process.reconstruction.darkref.darkrefscopy import DarkRefsCopy
from silx.gui import qt
import logging
import functools
import tempfile

_logger = logging.getLogger(__name__)


class DarkRefCopyProcessStack(FIFO, qt.QObject):
    """Implementation of the `.AxisProcess` but having a stack for treating
    scans and making computation in threads"""

    def __init__(self, process_id=None):
        self._save_dir = tempfile.mkdtemp()
        qt.QObject.__init__(self)
        FIFO.__init__(self, process_id=process_id)

    def __del__(self):
        try:
            shutil.rmtree(self._save_dir)
        except Exception as e:
            _logger.error(e)

    def _process(self, scan, configuration: dict, callback=None):
        if not isinstance(scan, TomwerScanBase):
            raise TypeError(f"{scan} is expected to be an instance of {TomwerScanBase}")
        if not isinstance(configuration, dict):
            raise TypeError(f"{configuration} is expected to be an instance of {dict}")
        ProcessManager().notify_dataset_state(
            dataset=scan,
            process=self,
            state=DatasetState.ON_GOING,
        )
        _logger.processStarted("dk-ref-copy {}" "".format(str(scan)))
        assert isinstance(scan, TomwerScanBase)
        self._scan_currently_computed = scan
        if isOnLbsram(scan) and isLowOnMemory(get_lbsram_path()) is True:
            # if computer is running into low memory on lbsram skip it
            mess = "low memory, skip dk-ref-copy", scan.path
            ProcessManager().notify_dataset_state(
                dataset=scan, process=self._process_id, state=DatasetState.SKIPPED
            )
            _logger.processSkipped(mess)
            if callback is not None:
                callback()
            self.scan_ready(scan=scan)
        else:
            self._scan_currently_computed = scan
            self._computationThread.init(scan=scan, inputs=configuration)
            # need to manage connect before starting it because
            fct_callback = functools.partial(self._end_threaded_computation, callback)
            self._computationThread.finished.connect(fct_callback)
            self._computationThread.start()

    def _end_computation(self, scan, future_scan, callback):
        """
        callback when the computation thread is finished

        :param scan: pass if no call to '_computationThread is made'
        """
        assert isinstance(scan, TomwerScanBase)
        FIFO._end_computation(
            self, scan=scan, future_scan=future_scan, callback=callback
        )

    def _end_threaded_computation(self, callback=None):
        assert self._scan_currently_computed is not None
        self._computationThread.finished.disconnect()
        if callback:
            callback()
        FIFO._end_threaded_computation(self)

    def _create_processing_thread(self, process_id=None) -> qt.QThread:
        return _ProcessingThread(process_id=process_id, save_dir=self._save_dir)


class _ProcessingThread(ProcessingThread, SuperviseProcess):
    """
    Thread use to execute the processing of the axis position
    """

    def __init__(self, save_dir, process_id=None):
        SuperviseProcess.__init__(self, process_id=process_id)
        try:
            ProcessingThread.__init__(self, process_id=process_id)
        except TypeError:
            ProcessingThread.__init__(self)
        self._save_dir = save_dir
        self._scan = None
        self._inputs = None

    def init(self, scan, inputs):
        self._scan = scan
        self._inputs = inputs

    def run(self):
        self.sigComputationStarted.emit()
        inputs = self._inputs
        inputs["data"] = self._scan
        inputs["save_dir"] = self._save_dir
        process = DarkRefsCopy(inputs=inputs, process_id=self.process_id)
        try:
            process.run()
        except Exception as e:
            _logger.warning(e)



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
"""
contains gui to select a slice in a volume
"""


__authors__ = [
    "H. Payno",
]

__license__ = "MIT"
__date__ = "26/02/2021"


from silx.gui import qt
from silx.io.url import DataUrl
from silx.gui.plot import PlotWindow
from typing import Union
from typing import Iterable
from silx.io.utils import get_data
from silx.gui.plot.utils.axis import SyncAxes
import numpy
import logging

from tomwer.core.process.reconstruction.scores.scores import ComputedScore

_logger = logging.getLogger(__name__)


class VignettesQDialog(qt.QDialog):
    """ """

    SIZE_HINT = qt.QSize(820, 820)

    def __init__(
        self,
        value_name,
        score_name,
        parent=None,
        value_format=None,
        score_format=None,
        colormap=None,
    ):
        qt.QDialog.__init__(self, parent)
        self._scan = None
        self._selectedValue = None
        self.setLayout(qt.QVBoxLayout())
        self._mainWidget = qt.QScrollArea(self)
        self._vignettesWidget = VignettesWidget(
            self,
            with_spacer=True,
            value_name=value_name,
            score_name=score_name,
            score_format=score_format,
            value_format=value_format,
            colormap=colormap,
        )
        self._mainWidget.setWidget(self._vignettesWidget)
        self.layout().addWidget(self._mainWidget)
        self._mainWidget.setWidgetResizable(True)

        # buttons
        types = qt.QDialogButtonBox.Ok | qt.QDialogButtonBox.Cancel
        self._buttons = qt.QDialogButtonBox(parent=self)
        self._buttons.setStandardButtons(types)
        self.layout().addWidget(self._buttons)

        # connect signal slot
        self._buttons.button(qt.QDialogButtonBox.Ok).clicked.connect(self.accept)

        self._buttons.button(qt.QDialogButtonBox.Cancel).clicked.connect(self.reject)

    def sizeHint(self):
        """Return a reasonable default size for usage in :class:`PlotWindow`"""
        return self.SIZE_HINT

    def setScores(self, scores, score_method):
        self._vignettesWidget.setScores(scores=scores, score_method=score_method)

    def selectedValue(self):
        if self._vignettesWidget is None:
            return self._selectedValue
        else:
            return self._vignettesWidget.selectedValue()

    def acccept(self):
        self._selectedValue = self._vignettesWidget.selectedValue()
        self._vignettesWidget.close()
        super().accept()

    def reject(self):
        self._selectedValue = self._vignettesWidget.selectedValue()
        self._vignettesWidget.close()
        super().reject()


class VignettesWidget(qt.QWidget):
    """
    Widget to display all the frames.

    :param QWidget parent:
    :param str value_name: name of the values for which we are looking for the
                           best one
    :param str score_name: name of the score computed
    :param score_format: None or str that can be formatted to display the
                         score
    :param value_format: None or str that can be formatted to display the
                         value
    """

    DEFAULT_PLOT_PER_ROW = 2

    def __init__(
        self,
        parent=None,
        value_name="value",
        score_name="score",
        with_spacer=True,
        value_format=None,
        score_format=None,
        colormap=None,
    ):
        qt.QWidget.__init__(self, parent)
        self._valueName = value_name
        self._scoreName = score_name
        self._nPlotPerRow = VignettesWidget.DEFAULT_PLOT_PER_ROW
        self._withSpacer = with_spacer
        self._vignettesGroup = qt.QButtonGroup()
        self.__constraintXAxis = None
        self.__constraintYAxis = None
        self._valueFormat = value_format
        self._scoreFormat = score_format
        self._colormap = colormap
        self._vignettes = []

    def close(self):
        for vignette in self._vignettes:
            vignette.close()
        super().close()

    def selectedValue(self):
        sel_vignette = self._vignettesGroup.checkedButton()
        if sel_vignette is not None:
            return sel_vignette.getValue()
        else:
            return None

    def setNElementsPerRow(self, n: int):
        self._nPlotPerRow = n

    def setScores(self, scores: dict, score_method):
        """
        Expect a dictionary with possible values to select as key and
        (2D numpy.array, score) as value.
        Where the 2D numpy.array is the frame to display and the score if the
        "indicator" score to display with the frame.
        :param dict scores: with score as key and url or numpy array as value
        """
        if len(scores) < 1:
            return
        self.setLayout(qt.QGridLayout())
        i_row = 0
        scores_values = []
        for i_score, (value, (data, score_cls)) in enumerate(scores.items()):
            if not isinstance(score_cls, ComputedScore):
                raise TypeError(
                    f"score is expected to be a dict with values as (v1: numpy.ndarray, v2: ComputedScore). v2 type Found: {type(score_cls)}"
                )
            scores_values.append(score_cls.get(score_method))
        highest_score_indices = numpy.nanargmax(scores_values)
        self._vignettesGroup = qt.QButtonGroup(self)
        self._vignettesGroup.setExclusive(True)

        if not isinstance(highest_score_indices, Iterable):
            highest_score_indices = (highest_score_indices,)

        xAxis = []
        yAxis = []

        for i_score, (value, (data, score_cls)) in enumerate(scores.items()):
            score = score_cls.get(score_method)
            i_column = i_score % self.DEFAULT_PLOT_PER_ROW
            # TODO: instead of having a binary color we could use
            # colormap
            # TODO: synchronize zooms
            if i_score == highest_score_indices or i_score in highest_score_indices:
                frame_color = qt.Qt.green
            else:
                frame_color = qt.Qt.lightGray
            widget = Vignette(
                parent=self,
                value_name=self._valueName,
                score_name=self._scoreName,
                value=value,
                data=data,
                score=score,
                frame_color=frame_color,
                value_format=self._valueFormat,
                score_format=self._scoreFormat,
                colormap=self._colormap,
            )
            widget.setAttribute(qt.Qt.WA_DeleteOnClose)
            xAxis.append(widget.getPlotWidget().getXAxis())
            yAxis.append(widget.getPlotWidget().getYAxis())

            self.layout().addWidget(widget, i_row, i_column)
            if i_column == self.DEFAULT_PLOT_PER_ROW - 1:
                i_row += 1
            if i_score == 0:
                # we cannot request all widget to keep the aspect ratio
                widget.setKeepDataAspectRatio(True)
            self._vignettesGroup.addButton(widget)
            self._vignettes.append(widget)

        if self._withSpacer:
            spacer = qt.QWidget(self)
            spacer.setSizePolicy(qt.QSizePolicy.Expanding, qt.QSizePolicy.Expanding)
            self.layout().addWidget(spacer, i_row + 1, self.DEFAULT_PLOT_PER_ROW - 1)

        # constrain axis synchronization
        self.__constraintXAxis = SyncAxes(
            xAxis,
            syncLimits=False,
            syncScale=True,
            syncDirection=True,
            syncCenter=True,
            syncZoom=True,
        )
        self.__constraintYAxis = SyncAxes(
            yAxis,
            syncLimits=False,
            syncScale=True,
            syncDirection=True,
            syncCenter=True,
            syncZoom=True,
        )


class _PlotForVignette(PlotWindow):
    def __init__(self, parent=None):
        PlotWindow.__init__(
            self,
            parent=parent,
            yInverted=True,
            copy=False,
            save=False,
            print_=False,
            control=False,
            mask=False,
        )
        self.setKeepDataAspectRatio(False)
        self.setAxesDisplayed(False)
        self.toolBar().hide()
        self.getInteractiveModeToolBar().hide()
        self.getOutputToolBar().hide()
        self.setInteractiveMode("zoom", zoomOnWheel=False)

    def close(self) -> bool:
        super().close()


class Vignette(qt.QToolButton):
    """Widget to display a vignette"""

    FRAME_WIDTH = 2

    def __init__(
        self,
        parent,
        value,
        value_name: str,
        score_name: str,
        data: Union[DataUrl, numpy.array],
        score: float,
        frame_color: qt.QColor,
        score_format=None,
        value_format=None,
        colormap=None,
    ):
        self._value = value
        self._scoreName = score_name
        self._valueName = value_name
        qt.QToolButton.__init__(self, parent)
        self.setCheckable(True)
        self.setLayout(qt.QVBoxLayout())
        self._plot = _PlotForVignette(parent=self)
        self._plot.setDefaultColormap(colormap=colormap)

        self.layout().addWidget(self._plot)
        self._valueLabel = ValueLabel(
            self,
            value=value,
            score=score,
            score_name=self._scoreName,
            value_name=self._valueName,
            value_format=value_format,
            score_format=score_format,
        )
        self.layout().addWidget(self._valueLabel)
        self.setFixedSize(400, 400)
        self._frameColor = frame_color
        self._selectedFrameColor = qt.Qt.black

        if isinstance(data, DataUrl):
            data = get_data(data)
        if data.ndim == 3 and data.shape[0] == 1:
            data = data.reshape(data.shape[1:])
        self._plot.addImage(data)

    def setKeepDataAspectRatio(self, keep):
        self._plot.setKeepDataAspectRatio(keep)

    def getPlotWidget(self):
        return self._plot

    def getValue(self):
        return self._value

    def paintEvent(self, event):
        super().paintEvent(event)
        painter = qt.QPainter(self)
        half_h_width = self.FRAME_WIDTH // 2
        rect = qt.QRect(
            half_h_width,
            half_h_width,
            self.width() - self.FRAME_WIDTH,
            self.height() - self.FRAME_WIDTH,
        )
        pen = qt.QPen()
        pen.setWidth(Vignette.FRAME_WIDTH)
        pen.setColor(self._frameColor)
        painter.setPen(pen)
        painter.drawRect(rect)
        if self.isChecked():
            pen.setColor(self._selectedFrameColor)
            pen.setStyle(qt.Qt.DashLine)
            pen.setDashOffset(0.2)
            painter.setPen(pen)
            painter.drawRect(rect)

    def close(self):
        self._plot.clear()
        self._plot.setAttribute(qt.Qt.WA_DeleteOnClose)
        self._plot.close()
        self._plot = None


class ValueLabel(qt.QWidget):
    """Display the value and the associated score"""

    def __init__(
        self, parent, value, score, value_name, score_name, value_format, score_format
    ):
        qt.QWidget.__init__(self, parent)
        self.setLayout(qt.QHBoxLayout())
        if value_format is not None:
            str_value = value_format.format(value)
        else:
            str_value = str(value)
        txt = "{}: {}".format(value_name, str_value)
        self._valueLabel = qt.QLabel(txt, self)
        self.layout().addWidget(self._valueLabel)
        if score_format is not None:
            str_score = score_format.format(score)
        else:
            str_score = str(score)
        txt = "({}: {})".format(score_name, str_score)
        self._scoreLabel = qt.QLabel(txt, self)
        self.layout().addWidget(self._scoreLabel)



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "11/12/2017"


import os
import shutil
import tempfile
import fabio
import numpy
from silx.gui.utils.testutils import TestCaseQt
from tomwer.core.utils.scanutils import MockEDF
from tomwer.core.process.reconstruction.darkref.darkrefs import DarkRefs
from tomwer.core.process.reconstruction.darkref.params import DKRFRP
from tomwer.core.process.reconstruction.darkref.params import Method
from tomwer.core.scan.scanfactory import ScanFactory
from tomwer.test.utils import UtilsTest


class TestDarkRefsBehavior(TestCaseQt):
    """Test that the Darks and reference are correctly computed from the
    DarksRefs class
    """

    def setUp(self):
        TestCaseQt.setUp(self)
        self.datasetsID = "test10"
        self.tmpDir = tempfile.mkdtemp()
        self.thRef = {}
        """number of theoretical ref file the algorithm should create"""
        self.thDark = {}
        """number of theoretical dark file the algorithm should create"""
        self.dataset_folder = os.path.join(self.tmpDir, self.datasetsID)

        dataDir = UtilsTest.getEDFDataset(self.datasetsID)
        shutil.copytree(dataDir, self.dataset_folder)
        files = os.listdir(self.dataset_folder)
        for _f in files:
            if _f.startswith(("refHST", "darkHST", "dark.edf")):
                os.remove(os.path.join(self.dataset_folder, _f))

        self.recons_params = DKRFRP()
        self.recons_params._set_remove_opt(False)

    def tearDown(self):
        self.qapp.processEvents()
        self.darkRef = None
        shutil.rmtree(self.tmpDir)
        TestCaseQt.tearDown(self)

    def testDarkCreation(self):
        """Test that the dark is correctly computed"""
        self.recons_params.ref_calc_method = Method.none
        self.recons_params.dark_calc_method = Method.median

        dar_ref_process = DarkRefs(
            inputs={
                "dark_ref_params": self.recons_params,
                "force_sync": True,
                "darkhst_prefix": "darkHST",
                "data": self.dataset_folder,
            }
        )
        dar_ref_process.run()
        self.qapp.processEvents()
        if os.path.basename(self.dataset_folder) == "test10":
            self.assertTrue("darkend0000.edf" in os.listdir(self.dataset_folder))
            self.assertTrue("dark.edf" in os.listdir(self.dataset_folder))
            self.assertEqual(
                len(
                    dar_ref_process.getDarkHSTFiles(
                        self.dataset_folder, prefix=self.recons_params.dark_prefix
                    )
                ),
                1,
            )
            self.assertEqual(
                len(
                    dar_ref_process.getRefHSTFiles(
                        self.dataset_folder, prefix=self.recons_params.ref_prefix
                    )
                ),
                0,
            )

    def testRefCreation(self):
        """Test that the dark is correctly computed"""
        self.recons_params.ref_calc_method = Method.median
        self.recons_params.dark_calc_method = Method.none

        dar_ref_process = DarkRefs(
            inputs={
                "dark_ref_params": self.recons_params,
                "force_sync": True,
                "darkhst_prefix": "darkHST",
                "data": self.dataset_folder,
            }
        )

        dar_ref_process.run()
        self.qapp.processEvents()
        self.assertTrue("darkend0000.edf" in os.listdir(self.dataset_folder))
        self.assertFalse("dark0000.edf" in os.listdir(self.dataset_folder))
        self.assertTrue("refHST0000.edf" in os.listdir(self.dataset_folder))
        self.assertTrue("refHST0020.edf" in os.listdir(self.dataset_folder))
        self.assertTrue("ref0000_0000.edf" in os.listdir(self.dataset_folder))
        self.assertTrue("ref0000_0020.edf" in os.listdir(self.dataset_folder))
        self.assertTrue("ref0001_0000.edf" in os.listdir(self.dataset_folder))
        self.assertTrue("ref0001_0020.edf" in os.listdir(self.dataset_folder))

    def testRemoveOption(self):
        """Test that the remove option is working"""
        self.recons_params.ref_calc_method = Method.none
        self.recons_params.dark_calc_method = Method.none
        self.recons_params._set_remove_opt(True)
        dar_ref_process = DarkRefs(
            inputs={
                "dark_ref_params": self.recons_params,
                "force_sync": True,
                "darkhst_prefix": "darkHST",
                "data": self.dataset_folder,
            }
        )
        dar_ref_process.run()
        self.qapp.processEvents()
        self.assertFalse("darkend0000.edf" in os.listdir(self.dataset_folder))
        self.assertFalse("dark0000.edf" in os.listdir(self.dataset_folder))
        self.assertFalse("refHST0000.edf" in os.listdir(self.dataset_folder))
        self.assertFalse("refHST0020.edf" in os.listdir(self.dataset_folder))
        self.assertFalse("ref0000_0000.edf" in os.listdir(self.dataset_folder))
        self.assertFalse("ref0000_0020.edf" in os.listdir(self.dataset_folder))
        self.assertFalse("ref0001_0000.edf" in os.listdir(self.dataset_folder))
        self.assertFalse("ref0001_0020.edf" in os.listdir(self.dataset_folder))

    def testSkipOption(self):
        """Test that the overwrite option is working"""
        self.recons_params.ref_calc_method = Method.none
        self.recons_params.dark_calc_method = Method.none
        self.recons_params._set_skip_if_exist(True)

        iniRefNFile = len(
            DarkRefs.getRefHSTFiles(
                self.dataset_folder, prefix=self.recons_params.ref_prefix
            )
        )
        iniDarkNFile = len(
            DarkRefs.getDarkHSTFiles(
                self.dataset_folder, prefix=self.recons_params.dark_prefix
            )
        )

        dar_ref_process = DarkRefs(
            inputs={
                "dark_ref_params": self.recons_params,
                "force_sync": True,
                "darkhst_prefix": "darkHST",
                "data": self.dataset_folder,
            }
        )
        dar_ref_process.run()
        self.qapp.processEvents()
        refs = DarkRefs.getRefHSTFiles(
            self.dataset_folder, prefix=self.recons_params.ref_prefix
        )
        darks = DarkRefs.getDarkHSTFiles(
            self.dataset_folder, prefix=self.recons_params.dark_prefix
        )
        self.assertTrue(len(refs) == iniRefNFile)
        self.assertTrue(len(darks) == iniDarkNFile)


class TestRefCalculationOneSerie(TestCaseQt):
    """
    Make sure the calculation is correct for the dark and flat field
    according to the method used.
    """

    def setUp(self):
        super().setUp()
        self.tmp_dir = tempfile.mkdtemp()
        n_scans = 5
        n_info = 1
        n_xml = 1
        MockEDF.fastMockAcquisition(self.tmp_dir, n_radio=n_scans)
        reFiles = {}
        data1 = numpy.zeros((20, 10))
        data2 = numpy.zeros((20, 10)) + 100
        reFiles["ref0000_0000.edf"] = data1
        reFiles["ref0001_0000.edf"] = data2
        reFiles["ref0002_0000.edf"] = data2
        reFiles["ref0003_0000.edf"] = data2
        for refFile in reFiles:
            file_desc = fabio.edfimage.EdfImage(data=reFiles[refFile])
            file_desc.write(os.path.join(self.tmp_dir, refFile))
        assert len(os.listdir(self.tmp_dir)) is (
            len(reFiles) + n_scans + n_xml + n_info
        )

        self.recons_params = DKRFRP()
        self.darkRef = DarkRefs(
            inputs={
                "dark_ref_params": self.recons_params,
                "data": self.tmp_dir,
            }
        )
        self.darkRef.setForceSync(True)
        self.recons_params.ref_pattern = "ref*.*[0-9]{3,4}_[0-9]{3,4}"

    def tearDown(self):
        shutil.rmtree(self.tmp_dir)
        super().tearDown()

    def testRefMedianCalculation(self):
        self.recons_params.ref_calc_method = Method.median
        self.recons_params.dark_calc_method = Method.none
        self.darkRef.run()
        refHST = os.path.join(self.tmp_dir, "refHST0000.edf")
        self.assertTrue(os.path.isfile(refHST))
        self.assertTrue(
            numpy.array_equal(fabio.open(refHST).data, numpy.zeros((20, 10)) + 100)
        )

    def testRefMeanCalculation(self):
        self.recons_params.ref_calc_method = Method.average
        self.recons_params.dark_calc_method = Method.none
        self.darkRef.run()
        refHST = os.path.join(self.tmp_dir, "refHST0000.edf")
        self.assertTrue(os.path.isfile(refHST))
        self.assertTrue(
            numpy.array_equal(fabio.open(refHST).data, numpy.zeros((20, 10)) + 75)
        )


class TestRefCalculationThreeSerie(TestCaseQt):
    """
    Make sure the calculation is correct for the dark and flat field
    according to the method used.
    """

    def setUp(self):
        super().setUp()
        self.tmp_dir = tempfile.mkdtemp()
        MockEDF.fastMockAcquisition(folder=self.tmp_dir, n_radio=1)
        reFiles = {}
        self.series = (0, 10, 200)
        for serie in self.series:
            data1 = numpy.zeros((20, 10)) + serie
            data2 = numpy.zeros((20, 10)) + 100 + serie
            reFiles["ref0000_" + str(serie).zfill(4) + ".edf"] = data1
            reFiles["ref0001_" + str(serie).zfill(4) + ".edf"] = data2
            reFiles["ref0002_" + str(serie).zfill(4) + ".edf"] = data2
            reFiles["ref0003_" + str(serie).zfill(4) + ".edf"] = data2
            for refFile in reFiles:
                file_desc = fabio.edfimage.EdfImage(data=reFiles[refFile])
                file_desc.write(os.path.join(self.tmp_dir, refFile))

        self.recons_params = DKRFRP()
        self.darkRef = DarkRefs(
            inputs={
                "dark_ref_params": self.recons_params,
                "data": self.tmp_dir,
                "force_sync": True,
            }
        )
        self.recons_params.ref_pattern = "ref*.*[0-9]{3,4}_[0-9]{3,4}"

    def tearDown(self):
        shutil.rmtree(self.tmp_dir)
        super().tearDown()

    def testRefMedianCalculation(self):
        self.recons_params.ref_calc_method = Method.median
        self.recons_params.dark_calc_method = Method.none
        self.darkRef.run()
        for serie in self.series:
            refHST = os.path.join(self.tmp_dir, "refHST" + str(serie).zfill(4) + ".edf")
            self.assertTrue(os.path.isfile(refHST))
            self.assertTrue(
                numpy.array_equal(
                    fabio.open(refHST).data, numpy.zeros((20, 10)) + 100 + serie
                )
            )

    def testRefMeanCalculation(self):
        self.recons_params.ref_calc_method = Method.average
        self.recons_params.dark_calc_method = Method.none
        self.darkRef.run()
        for serie in self.series:
            refHST = os.path.join(self.tmp_dir, "refHST" + str(serie).zfill(4) + ".edf")
            self.assertTrue(os.path.isfile(refHST))
            self.assertTrue(
                numpy.array_equal(
                    fabio.open(refHST).data, numpy.zeros((20, 10)) + 75 + serie
                )
            )


class TestDarkCalculationOneFrame(TestCaseQt):
    """Make sure computation of the Dark is correct"""

    def setUp(self):
        super().setUp()
        self.tmp_dir = tempfile.mkdtemp()
        n_scan = 1
        n_info = 1
        n_xml = 1
        MockEDF.fastMockAcquisition(self.tmp_dir, n_radio=n_scan)
        file_desc = fabio.edfimage.EdfImage(data=numpy.zeros((20, 10)) + 10)

        file_desc.write(os.path.join(self.tmp_dir, "darkend0000.edf"))
        assert len(os.listdir(self.tmp_dir)) is (1 + n_scan + n_info + n_xml)
        self.recons_params = DKRFRP()
        self.darkRef = DarkRefs(
            inputs={
                "dark_ref_params": self.recons_params,
                "force_sync": True,
                "data": self.tmp_dir,
            }
        )

    def tearDown(self):
        shutil.rmtree(self.tmp_dir)
        super().tearDown()

    def testDarkMeanCalculation(self):
        self.recons_params.ref_calc_method = Method.none
        self.recons_params.dark_calc_method = Method.average

        self.darkRef.run()
        refHST = os.path.join(self.tmp_dir, "dark.edf")
        self.assertTrue(os.path.isfile(refHST))
        self.assertTrue(
            numpy.array_equal(fabio.open(refHST).data, numpy.zeros((20, 10)) + 10)
        )

    def testDarkMedianCalculation(self):
        self.recons_params.ref_calc_method = Method.none
        self.recons_params.dark_calc_method = Method.median

        self.darkRef.run()
        refHST = os.path.join(self.tmp_dir, "dark.edf")
        self.assertTrue(os.path.isfile(refHST))

        self.assertTrue(
            numpy.array_equal(fabio.open(refHST).data, numpy.zeros((20, 10)) + 10)
        )


class TestDarkCalculation(TestCaseQt):
    """Make sure computation of the Dark is correct"""

    def setUp(self):
        super().setUp()
        self.tmp_dir = tempfile.mkdtemp()
        n_scan = 1
        n_xml = 1
        n_info = 1
        MockEDF.fastMockAcquisition(os.path.join(self.tmp_dir), n_radio=n_scan)

        file_desc = fabio.edfimage.EdfImage(data=numpy.zeros((20, 10)))
        file_desc.append_frame(data=(numpy.zeros((20, 10)) + 100))
        file_desc.append_frame(data=(numpy.zeros((20, 10)) + 100))
        file_desc.append_frame(data=(numpy.zeros((20, 10)) + 100))

        file_desc.write(os.path.join(self.tmp_dir, "darkend0000.edf"))
        assert len(os.listdir(self.tmp_dir)) is (1 + n_scan + n_xml + n_info)
        self.recons_params = DKRFRP()
        self.darkRef = DarkRefs(
            inputs={
                "data": self.tmp_dir,
                "dark_ref_params": self.recons_params,
                "force_sync": True,
            }
        )

    def tearDown(self):
        shutil.rmtree(self.tmp_dir)
        super().tearDown()

    def testDarkMeanCalculation(self):
        self.recons_params.ref_calc_method = Method.none
        self.recons_params.dark_calc_method = Method.average

        self.darkRef.run()
        refHST = os.path.join(self.tmp_dir, "dark.edf")
        self.assertTrue(os.path.isfile(refHST))
        self.assertTrue(
            numpy.array_equal(fabio.open(refHST).data, numpy.zeros((20, 10)) + 75)
        )

    def testDarkMedianCalculation(self):
        self.recons_params.ref_calc_method = Method.none
        self.recons_params.dark_calc_method = Method.median

        self.darkRef.run()
        refHST = os.path.join(self.tmp_dir, "dark.edf")
        self.assertTrue(os.path.isfile(refHST))

        self.assertTrue(
            numpy.array_equal(fabio.open(refHST).data, numpy.zeros((20, 10)) + 100)
        )


class TestDarkAccumulation(TestCaseQt):
    """
    Make sure computation for dark in accumulation are correct
    """

    def setUp(self):
        super().setUp()
        self.dataset = "bone8_1_"
        dataDir = UtilsTest.getEDFDataset(self.dataset)
        self.outputdir = tempfile.mkdtemp()
        shutil.copytree(src=dataDir, dst=os.path.join(self.outputdir, self.dataset))
        self.darkFile = os.path.join(self.outputdir, self.dataset, "dark.edf")
        # create a single 'bone8_1_' to be a valid acquisition directory
        MockEDF.fastMockAcquisition(os.path.join(self.outputdir, self.dataset))
        assert os.path.isfile(self.darkFile)
        with fabio.open(self.darkFile) as dsc:
            self.dark_reference = dsc.data
        # remove dark file
        os.remove(self.darkFile)

        self.recons_params = DKRFRP()
        self.recons_params.ref_calc_method = Method.none
        self.recons_params.dark_calc_method = Method.median
        self.recons_params.dark_pattern = "darkend*"
        self.recons_params.dark_prefix = "dark.edf"

    def tearDown(self):
        shutil.rmtree(self.outputdir)
        super().tearDown()

    def testComputation(self):
        """Test data bone8_1_ from id16b containing dark.edf of reference
        and darkend"""
        dark_ref_process = DarkRefs(
            inputs={
                "data": os.path.join(self.outputdir, self.dataset),
                "force_sync": True,
            }
        )

        dark_ref_process.run()
        self.assertTrue(os.path.isfile(self.darkFile))
        with fabio.open(self.darkFile) as dsc:
            self.computed_dark = dsc.data
        self.assertTrue(numpy.array_equal(self.computed_dark, self.dark_reference))


class TestPCOTomo(TestCaseQt):
    """Test processing of DKRF are correct"""

    def setUp(self):
        TestCaseQt.setUp(self)
        self.tmp_dir = tempfile.mkdtemp()
        MockEDF.fastMockAcquisition(self.tmp_dir)

        self.recons_params = DKRFRP()
        self.recons_params.ref_calc_method = Method.none
        self.recons_params.dark_calc_method = Method.none
        self.recons_params.dark_pattern = ".*_dark_.*"
        self.recons_params.ref_pattern = ".*_ref_.*"
        self.recons_params._set_remove_opt(True)

    def copyDataset(self, dataset):
        folder = os.path.join(self.tmp_dir, dataset)
        shutil.copytree(os.path.join(UtilsTest.getEDFDataset(dataset)), folder)
        self.scan = ScanFactory.create_scan_object(scan_path=folder)

    def tearDown(self):
        shutil.rmtree(self.tmp_dir)
        TestCaseQt.tearDown(self)

    def testDark3Scan(self):
        """
        Make sure the processing dark field for
        pcotomo_3scan_refdarkbeg_end_download are correct
        """
        self.dataset = "pcotomo_3scan_refdarkbeg_end_download"
        self.copyDataset(self.dataset)
        _file = os.path.join(
            self.tmp_dir, "pcotomo_3scan_refdarkbeg_end_download", "dark.edf"
        )
        if os.path.isfile(_file):
            os.remove(_file)
        self.recons_params.dark_calc_method = Method.median
        dar_ref_process = DarkRefs(
            inputs={
                "dark_ref_params": self.recons_params,
                "data": self.scan,
                "darkhst_prefix": "darkHST",
                "force_sync": True,
            }
        )
        dar_ref_process.run()
        darkHSTFiles = DarkRefs.getDarkHSTFiles(
            self.scan.path, prefix=self.recons_params.dark_prefix
        )
        self.assertEqual(len(darkHSTFiles), 2)
        dark0000 = os.path.join(self.scan.path, "dark0000.edf")
        dark1000 = os.path.join(self.scan.path, "dark1000.edf")
        self.assertTrue(dark0000 in darkHSTFiles)
        self.assertTrue(dark1000 in darkHSTFiles)

    def testRef3Scan(self):
        """
        Make sure the processing flat field for
        pcotomo_3scan_refdarkbeg_end_download are correct
        """
        self.dataset = "pcotomo_3scan_refdarkbeg_end_download"
        self.copyDataset(self.dataset)
        self.recons_params.ref_calc_method = Method.median
        dar_ref_process = DarkRefs(
            inputs={
                "dark_ref_params": self.recons_params,
                "data": self.scan,
                "darkhst_prefix": "darkHST",
                "force_sync": True,
            }
        )
        dar_ref_process.run()
        refHSTFiles = DarkRefs.getRefHSTFiles(
            self.scan.path, prefix=self.recons_params.ref_prefix
        )
        self.assertEqual(len(refHSTFiles), 2)
        f0000 = os.path.join(self.scan.path, "refHST0000.edf")
        self.assertTrue(f0000 in refHSTFiles)
        f1000 = os.path.join(self.scan.path, "refHST1000.edf")
        self.assertTrue(f1000 in refHSTFiles)

    def testDark2x2Scan(self):
        self.dataset = "pcotomo_2x2scan_refdarkbeg_end_conti"
        self.copyDataset(self.dataset)
        self.recons_params.dark_calc_method = Method.median
        dar_ref_process = DarkRefs(
            inputs={
                "dark_ref_params": self.recons_params,
                "data": self.scan,
                "darkhst_prefix": "darkHST",
                "force_sync": True,
            }
        )
        dar_ref_process.run()

    def testRef2x2Scan(self):
        self.dataset = "pcotomo_2x2scan_refdarkbeg_end_conti"
        self.copyDataset(self.dataset)
        self.recons_params.ref_calc_method = Method.median
        dar_ref_process = DarkRefs(
            inputs={
                "dark_ref_params": self.recons_params,
                "data": self.scan,
                "darkhst_prefix": "darkHST",
                "force_sync": True,
            }
        )
        dar_ref_process.run()



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
"""Set of icons for buttons.

Use :func:`getQIcon` to create Qt QIcon from the name identifying an icon.
"""

__authors__ = ["T. Vincent"]
__license__ = "MIT"
__date__ = "06/09/2017"


import os
import logging
import weakref
from silx.gui import qt
import tomwer.resources


_logger = logging.getLogger(__name__)
"""Module logger"""


_cached_icons = weakref.WeakValueDictionary()
"""Cache loaded icons in a weak structure"""


_supported_formats = None
"""Order of file format extension to check"""


def getQIcon(name):
    """Create a QIcon from its name.

    The resource name can be prefixed by the name of a resource directory. For
    example "silx:foo.png" identify the resource "foo.png" from the resource
    directory "silx".

    If no prefix are specified, the file with be returned from the silx
    resource directory with a specific path "gui/icons".

    See also :func:`silx.resources.register_resource_directory`.

    :param str name: Name of the icon, in one of the defined icons
                     in this module.
    :return: Corresponding QIcon
    :raises: ValueError when name is not known
    """
    qfile = getQFile(name)
    pixmap = qt.QPixmap(qfile.fileName())
    icon = qt.QIcon(pixmap)
    return icon


def getQPixmap(name):
    """Create a QPixmap from its name.

    The resource name can be prefixed by the name of a resource directory. For
    example "silx:foo.png" identify the resource "foo.png" from the resource
    directory "silx".

    If no prefix are specified, the file with be returned from the silx
    resource directory with a specific path "gui/icons".

    See also :func:`silx.resources.register_resource_directory`.

    :param str name: Name of the icon, in one of the defined icons
                     in this module.
    :return: Corresponding QPixmap
    :raises: ValueError when name is not known
    """
    qfile = getQFile(name)
    if qfile is None:
        raise ValueError("Not an illustration name: %s" % name)
    return qt.QPixmap(qfile.fileName())


def getQFile(name):
    """Create a QFile from an icon name. Filename is found
    according to supported Qt formats.

    The resource name can be prefixed by the name of a resource directory. For
    example "silx:foo.png" identify the resource "foo.png" from the resource
    directory "silx".

    If no prefix are specified, the file with be returned from the silx
    resource directory with a specific path "gui/icons".

    See also :func:`silx.resources.register_resource_directory`.

    :param str name: Name of the icon, in one of the defined icons
                     in this module.
    :return: Corresponding QFile
    :rtype: qt.QFile
    :raises: ValueError when name is not known
    """
    # if the extension is given in the name, skip the supported_formats
    _name = name.replace(" ", "_")
    if "." in _name:
        return _getQFile(_name)
    global _supported_formats
    if _supported_formats is None:
        _supported_formats = []
        supported_formats = qt.supportedImageFormats()
        order = ["mng", "gif", "svg", "png", "jpg"]
        for format_ in order:
            if format_ in supported_formats:
                _supported_formats.append(format_)
        if len(_supported_formats) == 0:
            _logger.error("No format supported for icons")
        else:
            _logger.debug("Format %s supported", ", ".join(_supported_formats))

    for format_ in _supported_formats:
        format_ = str(format_)
        out = _getQFile("%s.%s" % (_name, format_))
        if out and out.exists():
            return out
    raise ValueError("Not an illustration name: %s" % _name)


def getResourceFileName(name):
    return tomwer.resources._resource_filename(
        name, default_directory=os.path.join("gui", "illustrations")
    )


def _getQFile(name):
    filename = tomwer.resources._resource_filename(
        name, default_directory=os.path.join("gui", "illustrations")
    )
    qfile = qt.QFile(filename)
    if qfile.exists():
        return qfile
    else:
        return None



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import sys
import os
from silx.gui import qt
import argparse
from tomwer.gui.utils.splashscreen import getMainSplashScreen
from tomwer.gui.stacks import SliceStack
from tomwer.gui import icons
from tomwer.core.utils.resource import increase_max_number_file
import signal

logging.basicConfig(level=logging.WARNING)
_logger = logging.getLogger(__name__)


def getinputinfo():
    return "tomwer slicestack [scanDir]"


def sigintHandler(*args):
    """Handler for the SIGINT signal."""
    qt.QApplication.quit()


def addFolderAndSubFolder(stack, path):
    stack.addLeafScan(path)
    for f in os.listdir(path):
        _path = os.path.join(path, f)
        if os.path.isdir(_path) is True:
            addFolderAndSubFolder(stack, _path)


def main(argv):
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        "root_dir", help="Root dir to browse and to extract all slices files under it."
    )
    parser.add_argument(
        "--debug",
        dest="debug",
        action="store_true",
        default=False,
        help="Set logging system in debug mode",
    )

    options = parser.parse_args(argv[1:])

    increase_max_number_file()

    global app  # QApplication must be global to avoid seg fault on quit
    app = qt.QApplication.instance() or qt.QApplication([])

    qt.QLocale.setDefault(qt.QLocale(qt.QLocale.English))
    qt.QLocale.setDefault(qt.QLocale.c())
    signal.signal(signal.SIGINT, sigintHandler)
    sys.excepthook = qt.exceptionHandler
    timer = qt.QTimer()
    timer.start(500)
    # Application have to wake up Python interpreter, else SIGINT is not
    # catched
    timer.timeout.connect(lambda: None)

    splash = getMainSplashScreen()
    widget = SliceStack()
    widget.setWindowIcon(icons.getQIcon("tomwer"))
    options.root_dir = options.root_dir.rstrip(os.path.sep)
    addFolderAndSubFolder(stack=widget, path=options.root_dir)

    splash.finish(widget)
    widget.show()
    app.exec_()


if __name__ == "__main__":
    main(sys.argv)



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/


import os
import importlib
import sys

# pkg_resources is useful when this package is stored in a zip
# When pkg_resources is not available, the resources dir defaults to the
# directory containing this module.
try:
    import pkg_resources
except ImportError:
    pkg_resources = None

# For packaging purpose, patch this variable to use an alternative directory
# E.g., replace with _RESOURCES_DIR = '/usr/share/silx/data'
_RESOURCES_DIR = None

# For packaging purpose, patch this variable to use an alternative directory
# E.g., replace with _RESOURCES_DIR = '/usr/share/silx/doc'
# Not in use, uncomment when functionality is needed
# _RESOURCES_DOC_DIR = None

# cx_Freeze frozen support
# See http://cx-freeze.readthedocs.io/en/latest/faq.html#using-data-files
if getattr(sys, "frozen", False):
    # Running in a frozen application:
    # We expect resources to be located either in a silx/resources/ dir
    # relative to the executable or within this package.
    _dir = os.path.join(os.path.dirname(sys.executable), "tomwer", "resources")
    if os.path.isdir(_dir):
        _RESOURCES_DIR = _dir


class _ResourceDirectory(object):
    """Store a source of resources"""

    def __init__(self, package_name, package_path=None, forced_path=None):
        if forced_path is None:
            if package_path is None:
                if pkg_resources is None:
                    # In this case we have to compute the package path
                    # Else it will not be used
                    module = importlib.import_module(package_name)
                    package_path = os.path.abspath(os.path.dirname(module.__file__))
        self.package_name = package_name
        self.package_path = package_path
        self.forced_path = forced_path


_TOMWER_DIRECTORY = _ResourceDirectory(
    package_name=__name__,
    package_path=os.path.abspath(os.path.dirname(__file__)),
    forced_path=_RESOURCES_DIR,
)

_RESOURCE_DIRECTORIES = {}
_RESOURCE_DIRECTORIES["tomwer"] = _TOMWER_DIRECTORY


def _get_package_and_resource(resource, default_directory=None):
    """
    Return the resource directory class and a cleaned resource name without
    prefix.

    :param str: resource: Name of the resource with resource prefix.
    :param str default_directory: If the resource is not prefixed, the resource
        will be searched on this default directory of the silx resource
        directory.
    :rtype: tuple(_ResourceDirectory, str)
    :raises ValueError: If the resource name uses an unregistred resource
        directory name
    """
    if ":" in resource:
        prefix, resource = resource.split(":", 1)
    else:
        prefix = "tomwer"
        if default_directory is not None:
            resource = os.path.join(default_directory, resource)
    if prefix not in _RESOURCE_DIRECTORIES:
        raise ValueError("Resource '%s' uses an unregistred prefix", resource)
    resource_directory = _RESOURCE_DIRECTORIES[prefix]
    return resource_directory, resource


def _resource_filename(resource, default_directory=None):
    """Return filename corresponding to resource.

     The existence of the resource is not checked.

     The resource name can be prefixed by the name of a resource directory. For
    example "silx:foo.png" identify the resource "foo.png" from the resource
     directory "silx". See also :func:`register_resource_directory`.

     :param str resource: Resource path relative to resource directory
                         using '/' path separator. It can be either a file or
                          a directory.
     :param str default_directory: If the resource is not prefixed, the resource
         will be searched on this default directory of the silx resource
         directory. It should only be used internally by silx.
     :return: Absolute resource path in the file system
     :rtype: str
    """
    resource_directory, resource_name = _get_package_and_resource(
        resource, default_directory=default_directory
    )
    if resource_directory.forced_path is not None:
        # if set, use this directory
        base_dir = resource_directory.forced_path
        resource_path = os.path.join(base_dir, *resource_name.split("/"))
        return resource_path
    elif pkg_resources is None:
        # Fallback if pkg_resources is not available
        base_dir = resource_directory.package_path
        resource_path = os.path.join(base_dir, *resource_name.split("/"))
        return resource_path
    else:
        # Preferred way to get resources as it supports zipfile package
        package_name = resource_directory.package_name
        return pkg_resources.resource_filename(package_name, resource_name)
